==> nina-0.3.2-src.tar.gz: nina/java/translator/net/morilib/nina/cmd/Ninac.java <==
/*
 * Copyright 2009-2010 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.nina.cmd;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;

import net.morilib.nina.NinaCompoundException;
import net.morilib.nina.NinaException;
import net.morilib.nina.Quadro;
import net.morilib.nina.translate.NinaTranslator;
import net.morilib.nina.translate.NinaTranslatorJavaScript;

/**
 *
 *
 * @author MORIGUCHI, Yuichiro 2013/10/23
 */
public class Ninac {

	private static final Pattern POPT = Pattern.compile("-P([^=]+)=(.*)");
	private static final Pattern TABP = Pattern.compile("--tab=([0-9]+)");

	NinatOptions opts;

	int parseOptions(String[] args, Map<String, String> prms) {
		int k = 0, x;
		Matcher m;
		String s;

		for(; k < args.length; k++) {
			s = args[k];
			if(s.equals("-J")) {
				opts.jflag = true;
			} else if(s.startsWith("-L")) {
				opts.libs.add(s.substring(2));
			} else if(s.equals("-v")) {
				opts.console = opts;
			} else if(s.equals("-i")) {
				opts.setTranslator(new NinaTranslatorJavaScript());
				opts.opts.put("interpret", "on");
			} else if((m = POPT.matcher(s)).matches()) {
				prms.put(m.group(1), m.group(2));
			} else if(!s.startsWith("--")) {
				return k;
			} else if(s.equals("--help")) {
				opts.usage();
				System.exit(2);
			} else if(s.equals("--lint")) {
				Quadro.errst = System.err;
			} else if(s.startsWith("--libpath=")) {
				opts.libs.add(s.substring(10));
			} else if(s.equals("--verbose")) {
				opts.console = opts;
			} else if((m = TABP.matcher(s)).matches()) {
				Quadro.setTab(Integer.valueOf(m.group(1)));
			} else if((x = s.indexOf('=')) < 0) {
				opts.opts.put(s.substring(2), "");
			} else {
				opts.opts.put(s.substring(2, x), s.substring(x + 1));
			}
		}
		return k;
	}

	void process(String arg, Map<String, String> prms, boolean only,
			PrintStream pr) {
		InputStream ins = null;
		NinaTranslator tr;
		Reader rd = null;
		String s, t;
		int x;

		if(arg.endsWith("/")) {
			opts.perror("filenotfound", arg);
			return;
		} else if((x = arg.lastIndexOf('.')) < 0) {
			s = arg + ".nina";
			t = arg;
		} else {
			s = arg;
			t = arg.substring(0, x);
		}

		if((x = t.lastIndexOf('/')) < 0) {
			opts.filename = t;
		} else {
			opts.filename = t.substring(x + 1);
		}

		try {
			ins = new FileInputStream(s);
			rd  = new InputStreamReader(ins);
			tr  = opts.getTranslator();
			if(tr == null) {
				opts.perror("languagenotsupport");
				System.exit(2);
			}

			if(only) {
				tr.compile(s, rd, opts, prms);
			} else {
				tr.translate(s, rd, opts, opts.libs, prms, pr);
			}

			if(opts.isOptionDefined("statistics")) {
				tr.reportStatistics(System.out);
			}
		} catch(FileNotFoundException e) {
			opts.perror("filenotfound", s);
			System.exit(2);
		} catch(IOException e) {
			opts.perror("ioerror");
			if(opts.isOptionDefined("exception")) {
				e.printStackTrace(System.err);
			}
			System.exit(2);
		} catch(NinaCompoundException e) {
			for(NinaException e1 : e.getExceptions()) {
				opts.pmessage(e1.getMessage());
			}

			try {
				if(opts.isOptionDefined("exception")) {
					e.getCells().putBadfile(opts.filename + ".bad");
				} else if(opts.isOptionDefined("trace")) {
					e.getCells().putBadfile(opts.filename + ".bad");
				}
			} catch(IOException e1) {
				opts.perror("ioerror");
			}
			System.exit(2);
		} catch(NinaException e) {
			opts.pmessage(e.getMessage());
			if(opts.isOptionDefined("exception")) {
				e.printTrace(System.err);
				e.printStackTrace(System.err);
			} else if(opts.isOptionDefined("trace")) {
				e.printTrace(System.err);
			}
			System.exit(2);
		} finally {
			if(ins != null) {
				try {
					ins.close();
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
			}
		}
	}

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		Map<String, String> z;
		Ninac t = new Ninac();
		JavaCompiler cp;
		int k, x;

		z = new HashMap<String, String>();
		t.opts = new NinatOptions();
		if(args.length == 0) {
			t.opts.usage();
			System.exit(2);
		} else if((k = t.parseOptions(args, z)) == args.length) {
			t.opts.perror("nofiles");
			System.exit(2);
		} else if(t.opts.isOptionDefined("interpret")) {
			ByteArrayOutputStream ot;
			PrintStream pt;
			ScriptEngineManager mn;
			ScriptEngine en;

			try {
				ot = new ByteArrayOutputStream();
				pt = new PrintStream(ot, true);
				t.process(args[k], z, false, pt);
				mn = new ScriptEngineManager();
				en = mn.getEngineByName("javascript");
				en.eval(ot.toString());
				System.exit(0);
			} catch(ScriptException e) {
				throw new RuntimeException(e);
			}
		} else if(t.opts.isOptionDefined("only")) {
			for(; k < args.length; k++) {
				t.process(args[k], z, true, null);
			}
			System.exit(Quadro.isWarning() ? 1 : 0);
		} else {
			for(; k < args.length; k++) {
				t.process(args[k], z, false, null);
			}

			if(t.opts.jflag) {
				try {
					cp = ToolProvider.getSystemJavaCompiler();
					x  = cp.run(null, null, null,
							t.opts.getJavaFile().toString());
					System.exit(x);
				} catch(Exception e) {
					t.opts.perror("javacompiler");
					System.exit(4);
				}
			} else {
				System.exit(0);
			}
		}
	}

}

==> nina-0.4.0-src.tar.gz: nina/java/translator/net/morilib/nina/cmd/Ninac.java <==
/*
 * Copyright 2009-2010 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.nina.cmd;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;

import net.morilib.nina.NinaCompoundException;
import net.morilib.nina.NinaException;
import net.morilib.nina.Quadro;
import net.morilib.nina.translate.NinaTranslator;

/**
 *
 *
 * @author MORIGUCHI, Yuichiro 2013/10/23
 */
public class Ninac {

	private static void process(NinatOptions opts, NinaTranslator tr,
			String s, String arg, Map<String, String> prms,
			boolean only, PrintStream pr) {
		InputStream ins = null;
		Reader rd = null;

		try {
			ins = new FileInputStream(s);
			rd  = new InputStreamReader(ins);
			if(only) {
				tr.compile(s, rd, opts, prms);
			} else {
				tr.translate(s, rd, opts, opts.getLibraryList(), prms,
						pr);
			}

			if(opts.getReport() != null && opts.getReport() != tr) {
				process(opts, opts.getReport(), s, arg, prms, only,
						null);
			}

			if(opts.isOptionDefined("statistics")) {
				tr.reportStatistics(System.out);
			}
		} catch(FileNotFoundException e) {
			opts.perror("filenotfound", s);
			System.exit(2);
		} catch(IOException e) {
			opts.perror("ioerror");
			if(opts.isOptionDefined("exception")) {
				e.printStackTrace(System.err);
			}
			System.exit(2);
		} catch(NinaCompoundException e) {
			for(NinaException e1 : e.getExceptions()) {
				opts.pmessage(e1.getMessage());
			}

			try {
				if(opts.isOptionDefined("exception")) {
					e.getCells().putBadfile(opts.getFilename() + ".bad");
				} else if(opts.isOptionDefined("trace")) {
					e.getCells().putBadfile(opts.getFilename() + ".bad");
				}
			} catch(IOException e1) {
				opts.perror("ioerror");
			}
			System.exit(2);
		} catch(NinaException e) {
			opts.pmessage(e.getMessage());
			if(opts.isOptionDefined("exception")) {
				e.printTrace(System.err);
				e.printStackTrace(System.err);
			} else if(opts.isOptionDefined("trace")) {
				e.printTrace(System.err);
			}
			System.exit(2);
		} finally {
			if(ins != null) {
				try {
					ins.close();
				} catch (IOException e) {
					throw new RuntimeException(e);
				}
			}
		}
	}

	private static void process(NinatOptions opts, String arg,
			Map<String, String> prms, boolean only, PrintStream pr) {
		String s, t;
		int x;

		if(arg.endsWith("/")) {
			opts.perror("filenotfound", arg);
			return;
		} else if((x = arg.lastIndexOf('.')) < 0) {
			s = arg + ".nina";
			t = arg;
		} else {
			s = arg;
			t = arg.substring(0, x);
		}

		if((x = t.lastIndexOf('/')) < 0) {
			opts.filename = t;
		} else {
			opts.filename = t.substring(x + 1);
		}

		if(opts.getTranslator() == null) {
			opts.perror("languagenotsupport");
			System.exit(2);
		}
		process(opts, opts.getTranslator(), s, arg, prms, only, pr);
	}

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		NinatOptions opts = new NinatOptions();
		Map<String, String> z;
		JavaCompiler cp;
		int k, x;

		z = new HashMap<String, String>();
		opts = new NinatOptions();
		if(args.length == 0) {
			opts.usage();
			System.exit(2);
		} else if((k = opts.parseOptions(args, z)) == args.length) {
			opts.perror("nofiles");
			System.exit(2);
		} else if(opts.isOptionDefined("interpret")) {
			ByteArrayOutputStream ot;
			PrintStream pt;
			ScriptEngineManager mn;
			ScriptEngine en;

			try {
				ot = new ByteArrayOutputStream();
				pt = new PrintStream(ot, true);
				process(opts, args[k], z, false, pt);
				mn = new ScriptEngineManager();
				en = mn.getEngineByName("javascript");
				en.eval(ot.toString());
				System.exit(0);
			} catch(ScriptException e) {
				throw new RuntimeException(e);
			}
		} else if(opts.isOptionDefined("only")) {
			for(; k < args.length; k++) {
				process(opts, args[k], z, true, null);
			}
			System.exit(Quadro.isWarning() ? 1 : 0);
		} else {
			for(; k < args.length; k++) {
				process(opts, args[k], z, false, null);
			}

			if(opts.isJavaCompiler()) {
				try {
					cp = ToolProvider.getSystemJavaCompiler();
					x  = cp.run(null, null, null,
							opts.getJavaFile().toString());
					System.exit(x);
				} catch(Exception e) {
					opts.perror("javacompiler");
					System.exit(4);
				}
			} else {
				System.exit(0);
			}
		}
	}

}
